@@ -7,30 +7,41 @@ Virtual transport implementation.
 Emulates the AMQ API for non-AMQ transports.
 
 """
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals
 
 import base64
 import socket
+import sys
 import warnings
 
+from array import array
 from itertools import count
-from time import sleep, time
-from Queue import Empty
+from multiprocessing.util import Finalize
+from time import sleep
 
-from kombu.exceptions import StdChannelError
+from amqp.protocol import queue_declare_ok_t
+
+from kombu.exceptions import ResourceError, ChannelError
+from kombu.five import Empty, items, monotonic
 from kombu.utils import emergency_dump_state, say, uuid
 from kombu.utils.compat import OrderedDict
 from kombu.utils.encoding import str_to_bytes, bytes_to_str
-from kombu.utils.finalize import Finalize
 
 from kombu.transport import base
 
 from .scheduling import FairCycle
 from .exchange import STANDARD_EXCHANGE_TYPES
 
+ARRAY_TYPE_H = 'H' if sys.version_info[0] == 3 else b'H'
+
 UNDELIVERABLE_FMT = """\
-Message could not be delivered: No queues bound to exchange %(exchange)r
-using binding key %(routing_key)r
+Message could not be delivered: No queues bound to exchange {exchange!r} \
+using binding key {routing_key!r}.
+"""
+
+NOT_EQUIVALENT_FMT = """\
+Cannot redeclare exchange {0!r} in vhost {1!r} with \
+different type, durable, autodelete or arguments value.\
 """
 
 
@@ -103,12 +114,14 @@ class QoS(object):
         self._delivered = OrderedDict()
         self._delivered.restored = False
         self._dirty = set()
-        self._on_collect = Finalize(self,
-                                    self.restore_unacked_once,
-                                    exitpriority=1)
+        self._quick_ack = self._dirty.add
+        self._quick_append = self._delivered.__setitem__
+        self._on_collect = Finalize(
+            self, self.restore_unacked_once, exitpriority=1,
+        )
 
     def can_consume(self):
-        """Returns true if the channel can be consumed from.
+        """Return true if the channel can be consumed from.
 
         Used to ensure the client adhers to currently active
         prefetch limits.
@@ -121,7 +134,7 @@ class QoS(object):
         """Append message to transactional state."""
         if self._dirty:
             self._flush()
-        self._delivered[delivery_tag] = message
+        self._quick_append(delivery_tag, message)
 
     def get(self, delivery_tag):
         return self._delivered[delivery_tag]
@@ -139,35 +152,37 @@ class QoS(object):
 
     def ack(self, delivery_tag):
         """Acknowledge message and remove from transactional state."""
-        self._dirty.add(delivery_tag)
+        self._quick_ack(delivery_tag)
 
     def reject(self, delivery_tag, requeue=False):
         """Remove from transactional state and requeue message."""
         if requeue:
-            self.channel._restore(self._delivered[delivery_tag])
-        self._dirty.add(delivery_tag)
+            self.channel._restore_at_beginning(self._delivered[delivery_tag])
+        self._quick_ack(delivery_tag)
 
     def restore_unacked(self):
         """Restore all unacknowledged messages."""
         self._flush()
         delivered = self._delivered
         errors = []
+        restore = self.channel._restore
+        pop_message = delivered.popitem
 
         while delivered:
             try:
-                _, message = delivered.popitem()
+                _, message = pop_message()
             except KeyError:  # pragma: no cover
                 break
 
             try:
-                self.channel._restore(message)
-            except BaseException, exc:
+                restore(message)
+            except BaseException as exc:
                 errors.append((exc, message))
         delivered.clear()
         return errors
 
     def restore_unacked_once(self):
-        """Restores all unacknowledged message at shutdown/gc collect.
+        """Restores all unacknowledged messages at shutdown/gc collect.
 
         Will only be done once for each instance.
 
@@ -176,29 +191,25 @@ class QoS(object):
         self._flush()
         state = self._delivered
 
-        if not self.restore_at_shutdown:
+        if not self.restore_at_shutdown or not self.channel.do_restore:
             return
-        elif not self.channel.do_restore or getattr(state, 'restored', None):
+        if getattr(state, 'restored', None):
             assert not state
             return
-
         try:
             if state:
-                say('Restoring %r unacknowledged message(s).',
+                say('Restoring {0!r} unacknowledged message(s).',
                     len(self._delivered))
                 unrestored = self.restore_unacked()
 
                 if unrestored:
-                    errors, messages = zip(*unrestored)
-                    say('UNABLE TO RESTORE %s MESSAGES: %s',
+                    errors, messages = list(zip(*unrestored))
+                    say('UNABLE TO RESTORE {0} MESSAGES: {1}',
                         len(errors), errors)
                     emergency_dump_state(messages)
         finally:
             state.restored = True
 
-    def restore_visible(self, start=0, num=10, interval=10):
-        pass
-
 
 class Message(base.Message):
 
@@ -208,14 +219,16 @@ class Message(base.Message):
         body = payload.get('body')
         if body:
             body = channel.decode_body(body, properties.get('body_encoding'))
-        fields = {'body': body,
-                  'delivery_tag': properties['delivery_tag'],
-                  'content_type': payload.get('content-type'),
-                  'content_encoding': payload.get('content-encoding'),
-                  'headers': payload.get('headers'),
-                  'properties': properties,
-                  'delivery_info': properties.get('delivery_info'),
-                  'postencode': 'utf-8'}
+        fields = {
+            'body': body,
+            'delivery_tag': properties['delivery_tag'],
+            'content_type': payload.get('content-type'),
+            'content_encoding': payload.get('content-encoding'),
+            'headers': payload.get('headers'),
+            'properties': properties,
+            'delivery_info': properties.get('delivery_info'),
+            'postencode': 'utf-8',
+        }
         super(Message, self).__init__(channel, **dict(kwargs, **fields))
 
     def serializable(self):
@@ -225,11 +238,13 @@ class Message(base.Message):
         headers = dict(self.headers)
         # remove compression header
         headers.pop('compression', None)
-        return {'body': body,
-                'properties': props,
-                'content-type': self.content_type,
-                'content-encoding': self.content_encoding,
-                'headers': headers}
+        return {
+            'body': body,
+            'properties': props,
+            'content-type': self.content_type,
+            'content-encoding': self.content_encoding,
+            'headers': headers,
+        }
 
 
 class AbstractChannel(object):
@@ -269,9 +284,8 @@ class AbstractChannel(object):
     def _new_queue(self, queue, **kwargs):
         """Create new queue.
 
-        Some implementations needs to do additional actions when
-        the queue is created.  You can do so by overriding this
-        method.
+        Your transport can override this method if it needs
+        to do something whenever a new queue is declared.
 
         """
         pass
@@ -320,7 +334,7 @@ class Channel(AbstractChannel, base.StdChannel):
     body_encoding = 'base64'
 
     #: counter used to generate delivery tags for this channel.
-    _next_delivery_tag = count(1).next
+    _delivery_tags = count(1)
 
     #: Optional queue where messages with no route is delivered.
     #: Set by ``transport_options['deadletter_queue']``.
@@ -340,10 +354,17 @@ class Channel(AbstractChannel, base.StdChannel):
 
         # instantiate exchange types
         self.exchange_types = dict(
-            (typ, cls(self)) for typ, cls in self.exchange_types.items()
+            (typ, cls(self)) for typ, cls in items(self.exchange_types)
         )
 
-        self.channel_id = self.connection._next_channel_id()
+        try:
+            self.channel_id = self.connection._avail_channel_ids.pop()
+        except IndexError:
+            raise ResourceError(
+                'No free channel ids, current={0}, channel_max={1}'.format(
+                    len(self.connection.channels),
+                    self.connection.channel_max), (20, 10),
+            )
 
         topts = self.connection.client.transport_options
         for opt_name in self.from_transport_options:
@@ -357,24 +378,22 @@ class Channel(AbstractChannel, base.StdChannel):
                          nowait=False, passive=False):
         """Declare exchange."""
         type = type or 'direct'
-        exchange = exchange or 'amq.%s' % (type, )
+        exchange = exchange or 'amq.%s' % type
         if passive:
             if exchange not in self.state.exchanges:
-                raise StdChannelError(
-                    '404',
-                    u'NOT_FOUND - no exchange %r in vhost %r' % (
+                raise ChannelError(
+                    'NOT_FOUND - no exchange {0!r} in vhost {1!r}'.format(
                         exchange, self.connection.client.virtual_host or '/'),
-                    (50, 10), 'Channel.exchange_declare')
+                    (50, 10), 'Channel.exchange_declare', '404',
+                )
             return
         try:
             prev = self.state.exchanges[exchange]
             if not self.typeof(exchange).equivalent(prev, exchange, type,
                                                     durable, auto_delete,
                                                     arguments):
-                raise NotEquivalentError(
-                    'Cannot redeclare exchange %r in vhost %r with '
-                    'different type, durable or autodelete value' % (
-                        exchange, self.connection.client.virtual_host or '/'))
+                raise NotEquivalentError(NOT_EQUIVALENT_FMT.format(
+                    exchange, self.connection.client.virtual_host or '/'))
         except KeyError:
             self.state.exchanges[exchange] = {
                 'type': type,
@@ -394,16 +413,16 @@ class Channel(AbstractChannel, base.StdChannel):
         """Declare queue."""
         queue = queue or 'amq.gen-%s' % uuid()
         if passive and not self._has_queue(queue, **kwargs):
-            raise StdChannelError(
-                '404',
-                u'NOT_FOUND - no queue %r in vhost %r' % (
+            raise ChannelError(
+                'NOT_FOUND - no queue {0!r} in vhost {1!r}'.format(
                     queue, self.connection.client.virtual_host or '/'),
-                (50, 10), 'Channel.queue_declare')
+                (50, 10), 'Channel.queue_declare', '404',
+            )
         else:
             self._new_queue(queue, **kwargs)
-        return queue, self._size(queue), 0
+        return queue_declare_ok_t(queue, self._size(queue), 0)
 
-    def queue_delete(self, queue, if_unusued=False, if_empty=False, **kwargs):
+    def queue_delete(self, queue, if_unused=False, if_empty=False, **kwargs):
         """Delete queue."""
         if if_empty and self._size(queue):
             return
@@ -411,8 +430,9 @@ class Channel(AbstractChannel, base.StdChannel):
             exchange, routing_key, arguments = self.state.bindings[queue]
         except KeyError:
             return
-        meta = self.typeof(exchange).prepare_bind(queue, exchange,
-                                                  routing_key, arguments)
+        meta = self.typeof(exchange).prepare_bind(
+            queue, exchange, routing_key, arguments,
+        )
         self._delete(queue, exchange, *meta)
         self.state.bindings.pop(queue, None)
 
@@ -435,10 +455,9 @@ class Channel(AbstractChannel, base.StdChannel):
         exchange = exchange or 'amq.direct'
         table = self.state.exchanges[exchange].setdefault('table', [])
         self.state.bindings[queue] = exchange, routing_key, arguments
-        meta = self.typeof(exchange).prepare_bind(queue,
-                                                  exchange,
-                                                  routing_key,
-                                                  arguments)
+        meta = self.typeof(exchange).prepare_bind(
+            queue, exchange, routing_key, arguments,
+        )
         table.append(meta)
         if self.supports_fanout:
             self._queue_bind(exchange, *meta)
@@ -448,10 +467,9 @@ class Channel(AbstractChannel, base.StdChannel):
         raise NotImplementedError('transport does not support queue_unbind')
 
     def list_bindings(self):
-        for exchange in self.state.exchanges:
-            table = self.get_table(exchange)
-            for routing_key, pattern, queue in table:
-                yield queue, exchange, routing_key
+        return ((queue, exchange, rkey)
+                for exchange in self.state.exchanges
+                for rkey, pattern, queue in self.get_table(exchange))
 
     def queue_purge(self, queue, **kwargs):
         """Remove all ready messages from queue."""
@@ -459,14 +477,21 @@ class Channel(AbstractChannel, base.StdChannel):
 
     def basic_publish(self, message, exchange, routing_key, **kwargs):
         """Publish message."""
+        message['body'], body_encoding = self.encode_body(
+            message['body'], self.body_encoding,
+        )
         props = message['properties']
-        message['body'], props['body_encoding'] = \
-            self.encode_body(message['body'], self.body_encoding)
-        props['delivery_info']['exchange'] = exchange
-        props['delivery_info']['routing_key'] = routing_key
-        props['delivery_tag'] = self._next_delivery_tag()
-        self.typeof(exchange).deliver(message,
-                                      exchange, routing_key, **kwargs)
+        props.update(
+            body_encoding=body_encoding,
+            delivery_tag=next(self._delivery_tags),
+        )
+        props['delivery_info'].update(
+            exchange=exchange,
+            routing_key=routing_key,
+        )
+        self.typeof(exchange).deliver(
+            message, exchange, routing_key, **kwargs
+        )
 
     def basic_consume(self, queue, no_ack, callback, consumer_tag, **kwargs):
         """Consume from `queue`"""
@@ -547,20 +572,23 @@ class Channel(AbstractChannel, base.StdChannel):
     def _lookup(self, exchange, routing_key, default=None):
         """Find all queues matching `routing_key` for the given `exchange`.
 
-        Returns `default` if no queues matched.
+        Must return the string `default` if no queues matched.
 
         """
         if default is None:
             default = self.deadletter_queue
         try:
-            R = self.typeof(exchange).lookup(self.get_table(exchange),
-                                             exchange, routing_key, default)
+            R = self.typeof(exchange).lookup(
+                self.get_table(exchange),
+                exchange, routing_key, default,
+            )
         except KeyError:
             R = []
 
         if not R and default is not None:
-            warnings.warn(UndeliverableWarning(UNDELIVERABLE_FMT % {
-                'exchange': exchange, 'routing_key': routing_key}))
+            warnings.warn(UndeliverableWarning(UNDELIVERABLE_FMT.format(
+                exchange=exchange, routing_key=routing_key)),
+            )
             self._new_queue(default)
             R = [default]
         return R
@@ -570,10 +598,13 @@ class Channel(AbstractChannel, base.StdChannel):
         delivery_info = message.delivery_info
         message = message.serializable()
         message['redelivered'] = True
-        for queue in self._lookup(delivery_info['exchange'],
-                                  delivery_info['routing_key']):
+        for queue in self._lookup(
+                delivery_info['exchange'], delivery_info['routing_key']):
             self._put(queue, message)
 
+    def _restore_at_beginning(self, message):
+        return self._restore(message)
+
     def drain_events(self, timeout=None):
         if self._consumers and self.qos.can_consume():
             if hasattr(self, '_get_many'):
@@ -607,7 +638,7 @@ class Channel(AbstractChannel, base.StdChannel):
             is not implemented by the base virtual implementation.
 
         """
-        raise NotImplementedError('virtual channels does not support flow.')
+        raise NotImplementedError('virtual channels do not support flow.')
 
     def close(self):
         """Close channel, cancel all consumers, and requeue unacked
@@ -707,16 +738,21 @@ class Transport(base.Transport):
     #: Time to sleep between unsuccessful polls.
     polling_interval = 1.0
 
+    #: Max number of channels
+    channel_max = 65535
+
     def __init__(self, client, **kwargs):
         self.client = client
         self.channels = []
         self._avail_channels = []
         self._callbacks = {}
         self.cycle = self.Cycle(self._drain_channel, self.channels, Empty)
-        self._next_channel_id = count(1).next
         polling_interval = client.transport_options.get('polling_interval')
         if polling_interval is not None:
             self.polling_interval = polling_interval
+        self._avail_channel_ids = array(
+            ARRAY_TYPE_H, range(self.channel_max, 0, -1),
+        )
 
     def create_channel(self, connection):
         try:
@@ -728,6 +764,7 @@ class Transport(base.Transport):
 
     def close_channel(self, channel):
         try:
+            self._avail_channel_ids.append(channel.channel_id)
             try:
                 self.channels.remove(channel)
             except ValueError:
@@ -755,14 +792,14 @@ class Transport(base.Transport):
 
     def drain_events(self, connection, timeout=None):
         loop = 0
-        time_start = time()
+        time_start = monotonic()
         get = self.cycle.get
         polling_interval = self.polling_interval
         while 1:
             try:
                 item, channel = get(timeout=timeout)
             except Empty:
-                if timeout and time() - time_start >= timeout:
+                if timeout and monotonic() - time_start >= timeout:
                     raise socket.timeout()
                 loop += 1
                 if polling_interval is not None:
@@ -774,7 +811,7 @@ class Transport(base.Transport):
 
         if not queue or queue not in self._callbacks:
             raise KeyError(
-                "Received message for queue '%s' without consumers: %s" % (
+                'Message for queue {0!r} without consumers: {1}'.format(
                     queue, message))
 
         self._callbacks[queue](message)
